"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FanController = void 0;
const client_1 = require("../api/client");
function createDetailsBody() {
    return {
        'appVersion': 'V2.9.35 build3',
        'phoneBrand': 'HomeBridge-Vesync',
        'phoneOS': 'HomeBridge-Vesync',
        'traceId': Date.now()
    };
}
class FanController {
    constructor(fan, client) {
        this.fan = fan;
        this.client = client;
        this.details = {};
        this.initialize();
    }
    async initialize() {
        this.details = await this.getDetails();
    }
    setPower(on) {
        // this.details = on;
        const power = on ? 'on' : 'off';
        this.details.deviceStatus = power;
        this.client.put('131airPurifier/v1/device/deviceStatus', {
            'uuid': this.fan.uuid,
            'status': power
        });
    }
    isOn() {
        return this.details.deviceStatus === 'on';
    }
    getDetails() {
        return this.client.post('131airPurifier/v1/device/deviceDetail', {
            headers: {
                ...this.client.createHeaders()
            },
            json: {
                ...client_1.createBaseBody(),
                ...client_1.createAuthBody(this.client),
                ...createDetailsBody(),
                'uuid': this.fan.uuid
            }
        }).json();
    }
    setFanSpeed(speed) {
        this.details.level = speed;
        return this.client.put('131airPurifier/v1/device/updateSpeed', {
            'uuid': this.fan.uuid,
            'level': speed
        });
    }
    getFanSpeed() {
        return this.details.level;
    }
}
exports.FanController = FanController;
