"use strict";
const fanController_1 = require("./fan/fanController");
const client_1 = require("./api/client");
const client = new client_1.VesyncClient();
class LevoitAirPurifier {
    constructor(fan, log, accessory, api) {
        this.fan = fan;
        this.log = log;
        this.accessory = accessory;
        this.api = api;
        const fanController = new fanController_1.FanController(fan, client);
        const hap = api.hap;
        this.airPurifierService = this.getOrAddService(hap.Service.AirPurifier);
        // this.airPurifierService.getCharacteristic(hap.Characteristic.FilterLifeLevel)
        //     .on(CharacteristicEventTypes.GET, (callback: CharacteristicGetCallback) => {
        //         log.info('getting filter life level...');
        //         callback(null, 50);
        //     });
        this.airPurifierService.getCharacteristic(hap.Characteristic.Active)
            .onGet(() => {
            const isOn = fanController.isOn();
            return isOn ? hap.Characteristic.Active.ACTIVE : hap.Characteristic.Active.INACTIVE;
        })
            .onSet((value) => {
            const power = value === hap.Characteristic.Active.ACTIVE;
            fanController.setPower(power);
            return value;
        });
        this.airPurifierService.getCharacteristic(hap.Characteristic.CurrentAirPurifierState)
            .onGet(() => {
            return hap.Characteristic.CurrentAirPurifierState.PURIFYING_AIR;
        });
        this.airPurifierService.getCharacteristic(hap.Characteristic.TargetAirPurifierState)
            .onGet(() => {
            return hap.Characteristic.TargetAirPurifierState.AUTO;
        });
        this.airPurifierService.getCharacteristic(hap.Characteristic.RotationSpeed)
            .setProps({ minStep: 33, maxValue: 99 })
            .onGet(() => {
            const level = fanController.getFanSpeed();
            return level * 33;
        });
        this.airQualityService = this.getOrAddService(hap.Service.AirQualitySensor);
        this.airQualityService.getCharacteristic(hap.Characteristic.AirQuality)
            .onGet(() => {
            return hap.Characteristic.AirQuality.POOR;
        });
    }
    getOrAddService(service) {
        var _a;
        return (_a = this.accessory.getService(service)) !== null && _a !== void 0 ? _a : this.accessory.addService(service);
    }
}
class VesyncPlatform {
    constructor(log, config, api) {
        this.log = log;
        this.api = api;
        this.cachedAccessories = [];
        this.api.on('didFinishLaunching', async () => {
            await client.login(config.username, config.password);
            await this.findDevices();
        });
    }
    async findDevices() {
        const fans = await client.getDevices();
        fans.forEach(fan => {
            const cached = this.cachedAccessories.find(a => a.UUID === fan.uuid);
            if (cached) {
                this.log.debug('Restoring cached accessory: ' + cached.displayName);
                new LevoitAirPurifier(fan, this.log, cached, this.api);
            }
            else {
                this.log.debug('Creating new fan accessory...');
                const platformAccessory = new this.api.platformAccessory(fan.name, fan.uuid, 19 /* AIR_PURIFIER */);
                new LevoitAirPurifier(fan, this.log, platformAccessory, this.api);
                this.api.registerPlatformAccessories(PLUGIN_NAME, PLATFORM_NAME, [platformAccessory]);
            }
        });
    }
    /**
   * REQUIRED - Homebridge will call the "configureAccessory" method once for every cached
   * accessory restored
   */
    configureAccessory(accessory) {
        this.cachedAccessories.push(accessory);
    }
}
const PLUGIN_NAME = 'homebridge-vesync-client';
const PLATFORM_NAME = 'VesyncPlatform';
module.exports = (homebridge) => {
    homebridge.registerPlatform(PLATFORM_NAME, VesyncPlatform);
};
